/*
 * Copyright (c) 2020, Texas Instruments Incorporated
 * All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 *
 * *  Redistributions of source code must retain the above copyright
 *    notice, this list of conditions and the following disclaimer.
 *
 * *  Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in the
 *    documentation and/or other materials provided with the distribution.
 *
 * *  Neither the name of Texas Instruments Incorporated nor the names of
 *    its contributors may be used to endorse or promote products derived
 *    from this software without specific prior written permission.
 *
 * THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS"
 * AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO,
 * THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR
 * PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT OWNER OR
 * CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL,
 * EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO,
 * PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS;
 * OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY,
 * WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR
 * OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE,
 * EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 */

#include "ti_msp_dl_config.h"

uint16_t onTxPacket[] =  {0x0, 0x0, 0xE5F0, 0x1010, 0xE510, 0x10F0, 0xE510, 0xF010, 0xE510, 0x0010, 0xFFFF, 0xFFFF};

uint16_t offTxPacket[] = {0x0, 0x0, 0xE000, 0x0000, 0xE000, 0x0000, 0xE000, 0x0000, 0xE000, 0x0000, 0xFFFF, 0xFFFF};

uint16_t *txPacket;

int transmissionComplete = 0;
int idx = 0;

int message_len = sizeof(onTxPacket) / sizeof(onTxPacket[0]);

int main(void)
{
    SYSCFG_DL_init();

    while (1) {
        transmissionComplete = 0;
        idx = 1;
        txPacket = onTxPacket;
        NVIC_ClearPendingIRQ(SPI_0_INST_INT_IRQN);
        NVIC_EnableIRQ(SPI_0_INST_INT_IRQN);
        DL_SPI_transmitData16(SPI_0_INST, *txPacket);
        while (!transmissionComplete)
            __WFI();

        delay_cycles(16000000);


        // Block unti at least one button is held down
        while (DL_GPIO_readPins(
                BUTTONS_PORT, BUTTONS_SW1_PIN | BUTTONS_SW2_PIN | BUTTONS_SW3_PIN | BUTTONS_SW4_PIN) ==
                        (BUTTONS_SW1_PIN | BUTTONS_SW2_PIN | BUTTONS_SW3_PIN | BUTTONS_SW4_PIN) ) {
        }

        DL_TimerA_setCaptureCompareValue(PWM_0_INST, 4000, DL_TIMER_CC_0_INDEX);
        transmissionComplete = 0;
        idx = 1;
        txPacket = offTxPacket;
        NVIC_ClearPendingIRQ(SPI_0_INST_INT_IRQN);
        NVIC_EnableIRQ(SPI_0_INST_INT_IRQN);
        DL_SPI_transmitData16(SPI_0_INST, *txPacket);
        while (!transmissionComplete)
            __WFI();

        delay_cycles(16000000);
    }
}

void SPI_0_INST_IRQHandler(void)
{
    switch (DL_SPI_getPendingInterrupt(SPI_0_INST)) {
        case DL_SPI_IIDX_TX:
            DL_SPI_transmitData16(SPI_0_INST, txPacket[idx]);
            idx++;
            if (idx == message_len) {
               transmissionComplete = 1;
               NVIC_DisableIRQ(SPI_0_INST_INT_IRQN);
            }
            break;
        default:
            break;
    }
}
